@@ -5,7 +5,7 @@
 #  glpkAPI.R
 #  R interface to GLPK.
 #
-#  Copyright (C) 2011 Gabriel Gelius-Dietrich, Department for Bioinformatics,
+#  Copyright (C) 2011-2012 Gabriel Gelius-Dietrich, Dpt. for Bioinformatics,
 #  Institute for Informatics, Heinrich-Heine-University, Duesseldorf, Germany.
 #  All right reserved.
 #  Email: geliudie@uni-duesseldorf.de
@@ -33,7 +33,11 @@
 
 delProbGLPK <- function(lp) {
 
-    invisible(.Call("delProb", lp, PACKAGE = "glpkAPI"))
+    invisible(
+        .Call("delProb", PACKAGE = "glpkAPI",
+              ptr(lp)
+        )
+    )
 
 }
 
@@ -42,7 +46,11 @@ delProbGLPK <- function(lp) {
 
 eraseProbGLPK <- function(lp) {
 
-    invisible(.Call("eraseProb", lp, PACKAGE = "glpkAPI"))
+    invisible(
+        .Call("eraseProb", PACKAGE = "glpkAPI",
+              ptr(lp)
+        )
+    )
 
 }
 
@@ -53,7 +61,7 @@ copyProbGLPK <- function(lp, clp, name = GLP_OFF) {
 
     invisible(
         .Call("copyProb", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               clp,
               as.integer(name)
         )
@@ -64,10 +72,15 @@ copyProbGLPK <- function(lp, clp, name = GLP_OFF) {
 
 #------------------------------------------------------------------------------#
 
-initProbGLPK <- function() {
+initProbGLPK <- function(ptrtype = "glpk_prob") {
 
-    return(.Call("initProb", PACKAGE = "glpkAPI"))
+    lp <- .Call("initProb", PACKAGE = "glpkAPI",
+                as.character(ptrtype)
+          )
 
+    lpP <- glpk_Pointer(lp)
+
+    return(lpP)
 }
 
 
@@ -84,7 +97,7 @@ setProbNameGLPK <- function(lp, pname = NULL) {
 
     invisible(
         .Call("setProbName", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               Cpname
         )
     )
@@ -97,7 +110,7 @@ setProbNameGLPK <- function(lp, pname = NULL) {
 getProbNameGLPK <- function(lp) {
 
     pname <- .Call("getProbName", PACKAGE = "glpkAPI",
-                   lp
+                   ptr(lp)
              )
 
     return(pname)
@@ -118,7 +131,7 @@ setObjNameGLPK <- function(lp, oname = NULL) {
 
     invisible(
         .Call("setObjName", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               Coname
         )
     )
@@ -131,7 +144,7 @@ setObjNameGLPK <- function(lp, oname = NULL) {
 getObjNameGLPK <- function(lp) {
 
     oname <- .Call("getObjName", PACKAGE = "glpkAPI",
-                   lp
+                   ptr(lp)
              )
 
     return(oname)
@@ -145,7 +158,7 @@ createIndexGLPK <- function(lp) {
 
     invisible(
         .Call("createIndex", PACKAGE = "glpkAPI",
-              lp
+              ptr(lp)
         )
     )
 
@@ -158,7 +171,7 @@ deleteIndexGLPK <- function(lp) {
 
     invisible(
         .Call("deleteIndex", PACKAGE = "glpkAPI",
-              lp
+              ptr(lp)
         )
     )
 
@@ -344,7 +357,7 @@ setObjDirGLPK <- function(lp, lpdir) {
 
     invisible(
         .Call("setObjDir", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(lpdir)
         )
     )
@@ -357,7 +370,7 @@ setObjDirGLPK <- function(lp, lpdir) {
 getObjDirGLPK <- function(lp) {
 
     lpdir <- .Call("getObjDir", PACKAGE = "glpkAPI",
-                   lp
+                   ptr(lp)
                   )
     return(lpdir)
 
@@ -369,7 +382,7 @@ getObjDirGLPK <- function(lp) {
 addRowsGLPK <- function(lp, nrows) {
 
     frow <- .Call("addRows", PACKAGE = "glpkAPI",
-                  lp,
+                  ptr(lp),
                   as.integer(nrows)
              )
     return(frow)
@@ -390,7 +403,7 @@ setRowNameGLPK <- function(lp, i, rname = NULL) {
 
     invisible(
         .Call("setRowName", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               i,
               Crname
         )
@@ -404,7 +417,7 @@ setRowNameGLPK <- function(lp, i, rname = NULL) {
 getRowNameGLPK <- function(lp, i) {
 
     rname <- .Call("getRowName", PACKAGE = "glpkAPI",
-                   lp,
+                   ptr(lp),
                    i
              )
 
@@ -418,7 +431,7 @@ getRowNameGLPK <- function(lp, i) {
 findRowGLPK <- function(lp, rname) {
 
     rind <- .Call("findRow", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(rname)
         )
 
@@ -432,7 +445,7 @@ findRowGLPK <- function(lp, rname) {
 addColsGLPK <- function(lp, ncols) {
 
     fcol <- .Call("addCols", PACKAGE = "glpkAPI",
-                  lp,
+                  ptr(lp),
                   as.integer(ncols)
              )
 
@@ -454,7 +467,7 @@ setColNameGLPK <- function(lp, j, cname = NULL) {
 
     invisible(
         .Call("setColName", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               j,
               Ccname
         )
@@ -468,7 +481,7 @@ setColNameGLPK <- function(lp, j, cname = NULL) {
 getColNameGLPK <- function(lp, j) {
 
     cname <- .Call("getColName", PACKAGE = "glpkAPI",
-                   lp,
+                   ptr(lp),
                    j
              )
 
@@ -482,7 +495,7 @@ getColNameGLPK <- function(lp, j) {
 findColGLPK <- function(lp, cname) {
 
     cind <- .Call("findCol", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(cname)
         )
 
@@ -496,7 +509,7 @@ findColGLPK <- function(lp, cname) {
 getNumRowsGLPK <- function(lp) {
 
     nrows <- .Call("getNumRows", PACKAGE = "glpkAPI",
-                   lp
+                   ptr(lp)
              )
     return(nrows)
 
@@ -508,7 +521,7 @@ getNumRowsGLPK <- function(lp) {
 getNumColsGLPK <- function(lp) {
 
     ncols <- .Call("getNumCols", PACKAGE = "glpkAPI",
-                   lp
+                   ptr(lp)
                   )
     return(ncols)
 
@@ -528,7 +541,7 @@ setColsBndsGLPK <- function(lp, j, lb, ub, type = NULL) {
 
     invisible(
         .Call("setColsBnds", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               Ctype,
               as.numeric(lb),
@@ -553,7 +566,7 @@ setColsBndsObjCoefsGLPK <- function(lp, j, lb, ub, obj_coef, type = NULL) {
 
     invisible(
         .Call("setColsBndsObjCoefs", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               Ctype,
               as.numeric(lb),
@@ -570,7 +583,7 @@ setColBndGLPK <- function(lp, j, type, lb, ub) {
 
     invisible(
         .Call("setColBnd", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.integer(type),
               as.numeric(lb),
@@ -585,7 +598,7 @@ setColBndGLPK <- function(lp, j, type, lb, ub) {
 getColsLowBndsGLPK <- function(lp, j) {
 
     lowbnd <- .Call("getColsLowBnds", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
 
@@ -599,7 +612,7 @@ getColsLowBndsGLPK <- function(lp, j) {
 getColLowBndGLPK <- function(lp, j) {
 
     lowbnd <- .Call("getColLowBnd", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
     return(lowbnd)
@@ -612,7 +625,7 @@ getColLowBndGLPK <- function(lp, j) {
 getColsUppBndsGLPK <- function(lp, j) {
 
     uppbnd <- .Call("getColsUppBnds", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
 
@@ -626,7 +639,7 @@ getColsUppBndsGLPK <- function(lp, j) {
 getColUppBndGLPK <- function(lp, j) {
 
     uppbnd <- .Call("getColUppBnd", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
 
@@ -641,7 +654,7 @@ setColKindGLPK <- function(lp, j, kind) {
 
     invisible(
         .Call("setColKind", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.integer(kind)
         )
@@ -656,7 +669,7 @@ setColsKindGLPK <- function(lp, j, kind) {
 
     invisible(
         .Call("setColsKind", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.integer(kind)
         )
@@ -670,7 +683,7 @@ setColsKindGLPK <- function(lp, j, kind) {
 getColKindGLPK <- function(lp, j) {
 
     kind <- .Call("getColKind", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
 
@@ -684,7 +697,7 @@ getColKindGLPK <- function(lp, j) {
 getColsKindGLPK <- function(lp, j) {
 
     kind <- .Call("getColsKind", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
 
@@ -697,7 +710,9 @@ getColsKindGLPK <- function(lp, j) {
 
 getNumIntGLPK <- function(lp) {
 
-    num <- .Call("getNumInt", lp, PACKAGE = "glpkAPI")
+    num <- .Call("getNumInt", PACKAGE = "glpkAPI",
+                 ptr(lp)
+            )
 
     return(num)
 
@@ -708,7 +723,9 @@ getNumIntGLPK <- function(lp) {
 
 getNumBinGLPK <- function(lp) {
 
-    num <- .Call("getNumBin", lp, PACKAGE = "glpkAPI")
+    num <- .Call("getNumBin", PACKAGE = "glpkAPI",
+                 ptr(lp)
+            )
 
     return(num)
 
@@ -728,7 +745,7 @@ setRowsBndsGLPK <- function(lp, i, lb, ub, type = NULL) {
 
     invisible(
         .Call("setRowsBnds", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i),
               Ctype,
               as.numeric(lb),
@@ -745,7 +762,7 @@ setRhsZeroGLPK <- function(lp) {
 
     invisible(
         .Call("setRhsZero", PACKAGE = "glpkAPI",
-              lp
+              ptr(lp)
         )
     )
 
@@ -758,7 +775,7 @@ setRowBndGLPK <- function(lp, i, type, lb, ub) {
 
     invisible(
         .Call("setRowBnd", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i),
               as.integer(type),
               as.numeric(lb),
@@ -773,7 +790,7 @@ setRowBndGLPK <- function(lp, i, type, lb, ub) {
 getRowsLowBndsGLPK <- function(lp, i) {
 
     lowbnd <- .Call("getRowsLowBnds", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i)
         )
 
@@ -787,7 +804,7 @@ getRowsLowBndsGLPK <- function(lp, i) {
 getRowLowBndGLPK <- function(lp, i) {
 
     lowbnd <- .Call("getRowLowBnd", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i)
         )
     return(lowbnd)
@@ -800,7 +817,7 @@ getRowLowBndGLPK <- function(lp, i) {
 getRowsUppBndsGLPK <- function(lp, i) {
 
     uppbnd <- .Call("getRowsUppBnds", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i)
         )
 
@@ -814,7 +831,7 @@ getRowsUppBndsGLPK <- function(lp, i) {
 getRowUppBndGLPK <- function(lp, i) {
 
     uppbnd <- .Call("getRowUppBnd", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i)
         )
     return(uppbnd)
@@ -827,7 +844,7 @@ getRowUppBndGLPK <- function(lp, i) {
 getRowTypeGLPK <- function(lp, i) {
 
     type <- .Call("getRowType", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i)
         )
 
@@ -841,7 +858,7 @@ getRowTypeGLPK <- function(lp, i) {
 getColTypeGLPK <- function(lp, j) {
 
     type <- .Call("getColType", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
 
@@ -856,7 +873,7 @@ setObjCoefsGLPK <- function(lp, j, obj_coef) {
 
     invisible(
         .Call("setObjCoefs", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.numeric(obj_coef)
         )
@@ -871,7 +888,7 @@ setObjCoefGLPK <- function(lp, j, obj_coef) {
 
     invisible(
         .Call("setObjCoef", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.numeric(obj_coef)
         )
@@ -885,7 +902,7 @@ setObjCoefGLPK <- function(lp, j, obj_coef) {
 getObjCoefsGLPK <- function(lp, j) {
 
     obj_coef <- .Call("getObjCoefs", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
 
@@ -899,7 +916,7 @@ getObjCoefsGLPK <- function(lp, j) {
 getObjCoefGLPK <- function(lp, j) {
 
     obj_coef <- .Call("getObjCoef", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
     return(obj_coef)
@@ -913,7 +930,7 @@ loadMatrixGLPK <- function(lp, ne, ia, ja, ra) {
 
     invisible(
         .Call("loadMatrix", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(ne),
               as.integer(ia),
               as.integer(ja),
@@ -947,7 +964,7 @@ sortMatrixGLPK <- function(lp) {
 
     invisible(
         .Call("sortMatrix", PACKAGE = "glpkAPI",
-              lp
+              ptr(lp)
         )
     )
 
@@ -960,7 +977,7 @@ delRowsGLPK <- function(lp, nrows, i) {
 
     invisible(
         .Call("delRows", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(nrows),
               as.integer(i)
         )
@@ -975,7 +992,7 @@ delColsGLPK <- function(lp, ncols, j) {
 
     invisible(
         .Call("delCols", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(ncols),
               as.integer(j)
         )
@@ -990,7 +1007,7 @@ setRiiGLPK <- function(lp, i, rii) {
 
     invisible(
         .Call("setRii", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i),
               as.numeric(rii)
         )
@@ -1005,7 +1022,7 @@ setSjjGLPK <- function(lp, j, sjj) {
 
     invisible(
         .Call("setSjj", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.numeric(sjj)
         )
@@ -1019,7 +1036,7 @@ setSjjGLPK <- function(lp, j, sjj) {
 getRiiGLPK <- function(lp, i) {
 
     rii <- .Call("getRii", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i)
         )
     return(rii)
@@ -1032,7 +1049,7 @@ getRiiGLPK <- function(lp, i) {
 getSjjGLPK <- function(lp, j) {
 
     sjj <- .Call("getSjj", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j)
         )
     return(sjj)
@@ -1046,7 +1063,7 @@ scaleProbGLPK <- function(lp, opt) {
 
     invisible(
         .Call("scaleProb", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(opt)
         )
     )
@@ -1058,7 +1075,11 @@ scaleProbGLPK <- function(lp, opt) {
 
 unscaleProbGLPK <- function(lp) {
 
-    invisible(.Call("unscaleProb", lp, PACKAGE = "glpkAPI"))
+    invisible(
+        .Call("unscaleProb", PACKAGE = "glpkAPI",
+            ptr(lp)
+        )
+    )
 
 }
 
@@ -1069,7 +1090,7 @@ setRowStatGLPK <- function(lp, i, stat) {
 
     invisible(
         .Call("setRowStat", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i),
               as.integer(stat)
         )
@@ -1084,7 +1105,7 @@ setColStatGLPK <- function(lp, j, stat) {
 
     invisible(
         .Call("setColStat", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.integer(stat)
         )
@@ -1099,7 +1120,7 @@ stdBasisGLPK <- function(lp) {
 
     invisible(
         .Call("stdBasis", PACKAGE = "glpkAPI",
-              lp
+              ptr(lp)
         )
     )
 
@@ -1112,10 +1133,9 @@ advBasisGLPK <- function(lp) {
 
     invisible(
         .Call("advBasis", PACKAGE = "glpkAPI",
-              lp
+              ptr(lp)
         )
     )
-
 }
 
 
@@ -1125,10 +1145,9 @@ cpxBasisGLPK <- function(lp) {
 
     invisible(
         .Call("cpxBasis", PACKAGE = "glpkAPI",
-              lp
+              ptr(lp)
         )
     )
-
 }
 
 
@@ -1136,10 +1155,11 @@ cpxBasisGLPK <- function(lp) {
 
 warmUpGLPK <- function(lp) {
 
-    wup <- .Call("warmUp", lp, PACKAGE = "glpkAPI")
+    wup <- .Call("warmUp", PACKAGE = "glpkAPI",
+                 ptr(lp)
+    )
 
     return(wup)
-
 }
 
 
@@ -1152,7 +1172,6 @@ termOutGLPK <- function(flag) {
               as.integer(flag)
         )
     )
-
 }
 
 
@@ -1161,10 +1180,10 @@ termOutGLPK <- function(flag) {
 solveSimplexGLPK <- function(lp) {
 
     ret <- .Call("solveSimplex", PACKAGE = "glpkAPI",
-              lp
-        )
-    return(ret)
+              ptr(lp)
+           )
 
+    return(ret)
 }
 
 
@@ -1173,10 +1192,10 @@ solveSimplexGLPK <- function(lp) {
 solveSimplexExactGLPK <- function(lp) {
 
     ret <- .Call("solveSimplexExact", PACKAGE = "glpkAPI",
-              lp
-        )
-    return(ret)
+              ptr(lp)
+           )
 
+    return(ret)
 }
 
 
@@ -1185,10 +1204,10 @@ solveSimplexExactGLPK <- function(lp) {
 getObjValGLPK <- function(lp) {
 
     obj <- .Call("getObjVal", PACKAGE = "glpkAPI",
-              lp
-        )
-    return(obj)
+              ptr(lp)
+           )
 
+    return(obj)
 }
 
 
@@ -1197,10 +1216,10 @@ getObjValGLPK <- function(lp) {
 getSolStatGLPK <- function(lp) {
 
     stat <- .Call("getSolStat", PACKAGE = "glpkAPI",
-              lp
-        )
-    return(stat)
+              ptr(lp)
+            )
 
+    return(stat)
 }
 
 
@@ -1209,11 +1228,10 @@ getSolStatGLPK <- function(lp) {
 getColsPrimGLPK <- function(lp) {
 
     col_prim <- .Call("getColsPrim", PACKAGE = "glpkAPI",
-              lp
-        )
+                      ptr(lp)
+                )
 
     return(col_prim)
-
 }
 
 
@@ -1222,12 +1240,11 @@ getColsPrimGLPK <- function(lp) {
 getColPrimGLPK <- function(lp, j) {
 
     col_prim <- .Call("getColPrim", PACKAGE = "glpkAPI",
-              lp,
-              as.integer(j)
-        )
+                      ptr(lp),
+                      as.integer(j)
+                )
 
     return(col_prim)
-
 }
 
 
@@ -1235,10 +1252,11 @@ getColPrimGLPK <- function(lp, j) {
 
 getPrimStatGLPK <- function(lp) {
 
-    prim_stat <- .Call("getPrimStat", lp, PACKAGE = "glpkAPI")
+    prim_stat <- .Call("getPrimStat", PACKAGE = "glpkAPI",
+                       ptr(lp)
+                  )
 
     return(prim_stat)
-
 }
 
 
@@ -1246,10 +1264,11 @@ getPrimStatGLPK <- function(lp) {
 
 getDualStatGLPK <- function(lp) {
 
-    dual_stat <- .Call("getDualStat", lp, PACKAGE = "glpkAPI")
+    dual_stat <- .Call("getDualStat", PACKAGE = "glpkAPI",
+                       ptr(lp)
+                 )
 
     return(dual_stat)
-
 }
 
 
@@ -1258,12 +1277,11 @@ getDualStatGLPK <- function(lp) {
 getRowStatGLPK <- function(lp, i) {
 
     row_stat <- .Call("getRowStat", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(i)
                 )
 
     return(row_stat)
-
 }
 
 
@@ -1272,11 +1290,10 @@ getRowStatGLPK <- function(lp, i) {
 getRowsStatGLPK <- function(lp) {
 
     rows_stat <- .Call("getRowsStat", PACKAGE = "glpkAPI",
-              lp
-        )
+                       ptr(lp)
+                 )
 
     return(rows_stat)
-
 }
 
 
@@ -1285,12 +1302,11 @@ getRowsStatGLPK <- function(lp) {
 getRowPrimGLPK <- function(lp, i) {
 
     row_prim <- .Call("getRowPrim", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(i)
                 )
 
     return(row_prim)
-
 }
 
 
@@ -1299,8 +1315,8 @@ getRowPrimGLPK <- function(lp, i) {
 getRowsPrimGLPK <- function(lp) {
 
     rows_prim <- .Call("getRowsPrim", PACKAGE = "glpkAPI",
-              lp
-        )
+                       ptr(lp)
+                 )
 
     return(rows_prim)
 
@@ -1312,12 +1328,11 @@ getRowsPrimGLPK <- function(lp) {
 getRowDualGLPK <- function(lp, i) {
 
     row_dual <- .Call("getRowDual", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(i)
                 )
 
     return(row_dual)
-
 }
 
 
@@ -1330,7 +1345,6 @@ getRowsDualGLPK <- function(lp) {
         )
 
     return(rows_dual)
-
 }
 
 #------------------------------------------------------------------------------#
@@ -1338,12 +1352,11 @@ getRowsDualGLPK <- function(lp) {
 getColStatGLPK <- function(lp, j) {
 
     col_stat <- .Call("getColStat", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(j)
                 )
 
     return(col_stat)
-
 }
 
 
@@ -1352,10 +1365,10 @@ getColStatGLPK <- function(lp, j) {
 getColsStatGLPK <- function(lp) {
 
     cols_stat <- .Call("getColsStat", PACKAGE = "glpkAPI",
-              lp
-        )
-    return(cols_stat)
+                      ptr(lp) 
+                 )
 
+    return(cols_stat)
 }
 
 
@@ -1364,7 +1377,7 @@ getColsStatGLPK <- function(lp) {
 getColDualGLPK <- function(lp, j) {
 
     col_dual <- .Call("getColDual", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(j)
                 )
 
@@ -1378,20 +1391,21 @@ getColDualGLPK <- function(lp, j) {
 getColsDualGLPK <- function(lp) {
 
     cols_dual <- .Call("getColsDual", PACKAGE = "glpkAPI",
-              lp
-        )
-    return(cols_dual)
+                       ptr(lp)
+                 )
 
+    return(cols_dual)
 }
 
 #------------------------------------------------------------------------------#
 
 getUnbndRayGLPK <- function(lp) {
 
-    unbnd <- .Call("getUnbndRay", lp, PACKAGE = "glpkAPI")
+    unbnd <- .Call("getUnbndRay", PACKAGE = "glpkAPI",
+                   ptr(lp)
+             )
 
     return(unbnd)
-
 }
 
 
@@ -1400,10 +1414,10 @@ getUnbndRayGLPK <- function(lp) {
 solveInteriorGLPK <- function(lp) {
 
     ret <- .Call("solveInterior", PACKAGE = "glpkAPI",
-              lp
-        )
-    return(ret)
+              ptr(lp)
+           )
 
+    return(ret)
 }
 
 
@@ -1412,10 +1426,10 @@ solveInteriorGLPK <- function(lp) {
 getObjValIptGLPK <- function(lp) {
 
     obj <- .Call("getObjValIpt", PACKAGE = "glpkAPI",
-                 lp
+                 ptr(lp)
            )
-    return(obj)
 
+    return(obj)
 }
 
 
@@ -1424,10 +1438,10 @@ getObjValIptGLPK <- function(lp) {
 getSolStatIptGLPK <- function(lp) {
 
     stat <- .Call("getSolStatIpt", PACKAGE = "glpkAPI",
-                  lp
+                  ptr(lp)
             )
-    return(stat)
 
+    return(stat)
 }
 
 
@@ -1436,11 +1450,10 @@ getSolStatIptGLPK <- function(lp) {
 getColsPrimIptGLPK <- function(lp) {
 
     cols_prim <- .Call("getColsPrimIpt", PACKAGE = "glpkAPI",
-                       lp
+                       ptr(lp)
                  )
 
     return(cols_prim)
-
 }
 
 
@@ -1449,12 +1462,11 @@ getColsPrimIptGLPK <- function(lp) {
 getColPrimIptGLPK <- function(lp, j) {
 
     col_prim <- .Call("getColPrimIpt", PACKAGE = "glpkAPI",
-                       lp,
+                       ptr(lp),
                        as.integer(j)
                  )
 
     return(col_prim)
-
 }
 
 
@@ -1463,7 +1475,7 @@ getColPrimIptGLPK <- function(lp, j) {
 getRowPrimIptGLPK <- function(lp, i) {
 
     row_prim <- .Call("getRowPrimIpt", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(i)
                 )
 
@@ -1477,7 +1489,7 @@ getRowPrimIptGLPK <- function(lp, i) {
 getRowsPrimIptGLPK <- function(lp) {
 
     rows_prim <- .Call("getRowsPrimIpt", PACKAGE = "glpkAPI",
-                       lp
+                       ptr(lp)
                  )
 
     return(rows_prim)
@@ -1489,7 +1501,7 @@ getRowsPrimIptGLPK <- function(lp) {
 getRowDualIptGLPK <- function(lp, i) {
 
     row_dual <- .Call("getRowDualIpt", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(i)
                 )
 
@@ -1503,7 +1515,7 @@ getRowDualIptGLPK <- function(lp, i) {
 getRowsDualIptGLPK <- function(lp) {
 
     rows_dual <- .Call("getRowsDualIpt", PACKAGE = "glpkAPI",
-                       lp
+                       ptr(lp)
                  )
 
     return(rows_dual)
@@ -1516,12 +1528,11 @@ getRowsDualIptGLPK <- function(lp) {
 getColDualIptGLPK <- function(lp, j) {
 
     col_dual <- .Call("getColDualIpt", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(j)
                 )
 
     return(col_dual)
-
 }
 
 
@@ -1530,11 +1541,10 @@ getColDualIptGLPK <- function(lp, j) {
 getColsDualIptGLPK <- function(lp) {
 
     cols_dual <- .Call("getColsDualIpt", PACKAGE = "glpkAPI",
-                       lp
+                       ptr(lp)
                  )
 
     return(cols_dual)
-
 }
 
 
@@ -1542,10 +1552,11 @@ getColsDualIptGLPK <- function(lp) {
 
 solveMIPGLPK <- function(lp) {
 
-    ret <- .Call("solveMIP", lp, PACKAGE = "glpkAPI")
+    ret <- .Call("solveMIP", PACKAGE = "glpkAPI",
+                 ptr(lp)
+           )
 
     return(ret)
-
 }
 
 
@@ -1553,10 +1564,11 @@ solveMIPGLPK <- function(lp) {
 
 mipStatusGLPK <- function(lp) {
 
-    stat <- .Call("mipStatus", lp, PACKAGE = "glpkAPI")
+    stat <- .Call("mipStatus", PACKAGE = "glpkAPI",
+                  ptr(lp)
+            )
 
     return(stat)
-
 }
 
 
@@ -1564,10 +1576,11 @@ mipStatusGLPK <- function(lp) {
 
 mipObjValGLPK <- function(lp) {
 
-    obj_val <- .Call("mipObjVal", lp, PACKAGE = "glpkAPI")
+    obj_val <- .Call("mipObjVal", PACKAGE = "glpkAPI",
+                     ptr(lp)
+               )
 
     return(obj_val)
-
 }
 
 
@@ -1576,12 +1589,11 @@ mipObjValGLPK <- function(lp) {
 mipRowValGLPK <- function(lp, i) {
 
     row_val <- .Call("mipRowVal", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(i)
                 )
 
     return(row_val)
-
 }
 
 
@@ -1589,10 +1601,11 @@ mipRowValGLPK <- function(lp, i) {
 
 mipRowsValGLPK <- function(lp) {
 
-    row_val <- .Call("mipRowsVal", lp, PACKAGE = "glpkAPI")
+    row_val <- .Call("mipRowsVal", PACKAGE = "glpkAPI",
+                     ptr(lp)
+               )
 
     return(row_val)
-
 }
 
 
@@ -1601,12 +1614,11 @@ mipRowsValGLPK <- function(lp) {
 mipColValGLPK <- function(lp, j) {
 
     col_val <- .Call("mipColVal", PACKAGE = "glpkAPI",
-                      lp,
+                      ptr(lp),
                       as.integer(j)
                 )
 
     return(col_val)
-
 }
 
 
@@ -1614,10 +1626,11 @@ mipColValGLPK <- function(lp, j) {
 
 mipColsValGLPK <- function(lp) {
 
-    col_val <- .Call("mipColsVal", lp, PACKAGE = "glpkAPI")
+    col_val <- .Call("mipColsVal", PACKAGE = "glpkAPI",
+                     ptr(lp)
+               )
 
     return(col_val)
-
 }
 
 
@@ -1626,11 +1639,10 @@ mipColsValGLPK <- function(lp) {
 getNumNnzGLPK <- function(lp) {
 
     nnz <- .Call("getNumNnz", PACKAGE = "glpkAPI",
-              lp
-        )
+                 ptr(lp)
+           )
 
     return(nnz)
-
 }
 
 
@@ -1639,11 +1651,11 @@ getNumNnzGLPK <- function(lp) {
 getMatRowGLPK <- function(lp, i) {
 
     row_val <- .Call("getMatRow", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i)
         )
-    return(row_val)
 
+    return(row_val)
 }
 
 
@@ -1666,7 +1678,7 @@ setMatRowGLPK <- function(lp, i, len, ind, val) {
     }
 
     check <- .Call("setMatRow", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(i),
               as.integer(len),
               Cind,
@@ -1674,7 +1686,6 @@ setMatRowGLPK <- function(lp, i, len, ind, val) {
     )
 
     return(check)
-
 }
 
 
@@ -1683,11 +1694,11 @@ setMatRowGLPK <- function(lp, i, len, ind, val) {
 getMatColGLPK <- function(lp, j) {
 
     row_val <- .Call("getMatCol", PACKAGE = "glpkAPI",
-              lp,
-              as.integer(j)
-        )
-    return(row_val)
+                     ptr(lp),
+                     as.integer(j)
+               )
 
+    return(row_val)
 }
 
 
@@ -1710,7 +1721,7 @@ setMatColGLPK <- function(lp, j, len, ind, val) {
     }
 
     check <- .Call("setMatCol", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(j),
               as.integer(len),
               Cind,
@@ -1727,7 +1738,7 @@ setMatColGLPK <- function(lp, j, len, ind, val) {
 readMPSGLPK <- function(lp, fmt, fname) {
 
     check <- .Call("readMPS", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(fmt),
               as.character(fname)
         )
@@ -1741,7 +1752,7 @@ readMPSGLPK <- function(lp, fmt, fname) {
 readLPGLPK <- function(lp, fname) {
 
     check <- .Call("readLP", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1754,7 +1765,7 @@ readLPGLPK <- function(lp, fname) {
 readProbGLPK <- function(lp, fname) {
 
     check <- .Call("readProb", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1767,7 +1778,7 @@ readProbGLPK <- function(lp, fname) {
 writeMPSGLPK <- function(lp, fmt, fname) {
 
     check <- .Call("writeMPS", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(fmt),
               as.character(fname)
         )
@@ -1781,7 +1792,7 @@ writeMPSGLPK <- function(lp, fmt, fname) {
 writeLPGLPK <- function(lp, fname) {
 
     check <- .Call("writeLP", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1794,7 +1805,7 @@ writeLPGLPK <- function(lp, fname) {
 writeProbGLPK <- function(lp, fname) {
 
     check <- .Call("writeProb", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1807,7 +1818,7 @@ writeProbGLPK <- function(lp, fname) {
 printSolGLPK <- function(lp, fname) {
 
     check <- .Call("printSol", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1820,7 +1831,7 @@ printSolGLPK <- function(lp, fname) {
 readSolGLPK <- function(lp, fname) {
 
     check <- .Call("readSol", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1833,7 +1844,7 @@ readSolGLPK <- function(lp, fname) {
 writeSolGLPK <- function(lp, fname) {
 
     check <- .Call("writeSol", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1846,7 +1857,7 @@ writeSolGLPK <- function(lp, fname) {
 printIptGLPK <- function(lp, fname) {
 
     check <- .Call("printIpt", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1859,7 +1870,7 @@ printIptGLPK <- function(lp, fname) {
 readIptGLPK <- function(lp, fname) {
 
     check <- .Call("readIpt", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1872,7 +1883,7 @@ readIptGLPK <- function(lp, fname) {
 writeIptGLPK <- function(lp, fname) {
 
     check <- .Call("writeIpt", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1885,7 +1896,7 @@ writeIptGLPK <- function(lp, fname) {
 printMIPGLPK <- function(lp, fname) {
 
     check <- .Call("printMIP", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1898,7 +1909,7 @@ printMIPGLPK <- function(lp, fname) {
 readMIPGLPK <- function(lp, fname) {
 
     check <- .Call("readMIP", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1911,7 +1922,7 @@ readMIPGLPK <- function(lp, fname) {
 writeMIPGLPK <- function(lp, fname) {
 
     check <- .Call("writeMIP", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.character(fname)
         )
     return(check)
@@ -1933,9 +1944,11 @@ versionGLPK <- function() {
 
 bfExistsGLPK <- function(lp) {
 
-    check <- .Call("bfExists", lp, PACKAGE = "glpkAPI")
-    return(check)
+    check <- .Call("bfExists", PACKAGE = "glpkAPI",
+                   ptr(lp)
+             )
 
+    return(check)
 }
 
 
@@ -1943,9 +1956,11 @@ bfExistsGLPK <- function(lp) {
 
 factorizeGLPK <- function(lp) {
 
-    check <- .Call("factorize", lp, PACKAGE = "glpkAPI")
-    return(check)
+    check <- .Call("factorize", PACKAGE = "glpkAPI",
+                   ptr(lp)    
+             )
 
+    return(check)
 }
 
 
@@ -1953,9 +1968,11 @@ factorizeGLPK <- function(lp) {
 
 bfUpdatedGLPK <- function(lp) {
 
-    check <- .Call("bfUpdated", lp, PACKAGE = "glpkAPI")
-    return(check)
+    check <- .Call("bfUpdated", PACKAGE = "glpkAPI",
+                   ptr(lp)    
+             )
 
+    return(check)
 }
 
 
@@ -1993,7 +2010,7 @@ setBfcpGLPK <- function(lp, parm, val) {
 
     invisible(
         .Call("setBfcp", PACKAGE = "glpkAPI",
-              lp,
+              ptr(lp),
               as.integer(npari),
               parmi,
               vali,
@@ -2002,7 +2019,6 @@ setBfcpGLPK <- function(lp, parm, val) {
               vald
         )
     )
-
 }
 
 
@@ -2011,11 +2027,10 @@ setBfcpGLPK <- function(lp, parm, val) {
 getBfcpGLPK <- function(lp) {
 
     parmB <- .Call("getBfcp", PACKAGE = "glpkAPI",
-                   lp
-    )
+                   ptr(lp)
+             )
 
     return(parmB)
-
 }
 
 
@@ -2024,12 +2039,11 @@ getBfcpGLPK <- function(lp) {
 getBheadGLPK <- function(lp, k) {
 
     bh <- .Call("getBhead", PACKAGE = "glpkAPI",
-                lp,
+                ptr(lp),
                 as.integer(k)
     )
 
     return(bh)
-
 }
 
 
@@ -2038,12 +2052,11 @@ getBheadGLPK <- function(lp, k) {
 getRbindGLPK <- function(lp, i) {
 
     rh <- .Call("getRbind", PACKAGE = "glpkAPI",
-                lp,
+                ptr(lp),
                 as.integer(i)
     )
 
     return(rh)
-
 }
 
 
@@ -2052,12 +2065,11 @@ getRbindGLPK <- function(lp, i) {
 getCbindGLPK <- function(lp, j) {
 
     ch <- .Call("getCbind", PACKAGE = "glpkAPI",
-                lp,
+                ptr(lp),
                 as.integer(j)
     )
 
     return(ch)
-
 }
 
 
@@ -2073,31 +2085,39 @@ printRangesGLPK <- function(lp, numrc = 0, rowcol = NULL, fname = "sar.txt") {
     }
 
     sensit <- .Call("printRanges", PACKAGE = "glpkAPI",
-                    lp,
+                    ptr(lp),
                     as.integer(numrc),
                     Crowcol,
                     as.character(fname)
               )
 
     return(sensit)
-
 }
 
 
 #------------------------------------------------------------------------------#
 
-mplAllocWkspGLPK <- function() {
+mplAllocWkspGLPK <- function(ptrtype = "tr_wksp") {
 
-    return(.Call("mplAllocWksp", PACKAGE = "glpkAPI"))
+    wk <- .Call("mplAllocWksp", PACKAGE = "glpkAPI",
+                as.character(ptrtype)
+          )
 
+    wkP <- trwks_Pointer(wk)
+
+    return(wkP)
 }
 
 
 #------------------------------------------------------------------------------#
 
-mplFreeWkspGLPK <- function(wksp) {
+mplFreeWkspGLPK <- function(wk) {
 
-    invisible(.Call("mplFreeWksp", wksp, PACKAGE = "glpkAPI"))
+    invisible(
+        .Call("mplFreeWksp", PACKAGE = "glpkAPI",
+              ptr(wk)
+        )
+    )
 
 }
 
@@ -2107,12 +2127,12 @@ mplFreeWkspGLPK <- function(wksp) {
 mplReadModelGLPK <- function(wk, fname, skip) {
 
     check <- .Call("mplReadModel", PACKAGE = "glpkAPI",
-              wk,
+              ptr(wk),
               as.character(fname),
               as.integer(skip)
         )
-    return(check)
 
+    return(check)
 }
 
 
@@ -2121,11 +2141,11 @@ mplReadModelGLPK <- function(wk, fname, skip) {
 mplReadDataGLPK <- function(wk, fname) {
 
     check <- .Call("mplReadData", PACKAGE = "glpkAPI",
-              wk,
+              ptr(wk),
               as.character(fname)
         )
-    return(check)
 
+    return(check)
 }
 
 
@@ -2141,11 +2161,11 @@ mplGenerateGLPK <- function(wk, fname = NULL) {
     }
 
     check <- .Call("mplGenerate", PACKAGE = "glpkAPI",
-              wk,
+              ptr(wk),
               Cfname
         )
-    return(check)
 
+    return(check)
 }
 
 
@@ -2153,8 +2173,12 @@ mplGenerateGLPK <- function(wk, fname = NULL) {
 
 mplBuildProbGLPK <- function(wk, lp) {
 
-    invisible(.Call("mplBuildProb", PACKAGE = "glpkAPI", wk, lp))
-
+    invisible(
+        .Call("mplBuildProb", PACKAGE = "glpkAPI",
+              ptr(wk),
+              ptr(lp)
+        )
+    )
 }
 
 
@@ -2163,12 +2187,12 @@ mplBuildProbGLPK <- function(wk, lp) {
 mplPostsolveGLPK <- function(wk, lp, sol) {
 
     check <- .Call("mplPostsolve", PACKAGE = "glpkAPI",
-              wk,
-              lp,
+              ptr(wk),
+              ptr(lp),
               as.integer(sol)
         )
-    return(check)
 
+    return(check)
 }
 
 
